"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageReceiver = void 0;
const tsyringe_1 = require("tsyringe");
const error_1 = require("../error");
const connections_1 = require("../modules/connections");
const DidRepository_1 = require("../modules/dids/repository/DidRepository");
const problem_reports_1 = require("../modules/problem-reports");
const JsonTransformer_1 = require("../utils/JsonTransformer");
const MessageValidator_1 = require("../utils/MessageValidator");
const messageType_1 = require("../utils/messageType");
const AgentConfig_1 = require("./AgentConfig");
const Dispatcher_1 = require("./Dispatcher");
const EnvelopeService_1 = require("./EnvelopeService");
const MessageSender_1 = require("./MessageSender");
const TransportService_1 = require("./TransportService");
const helpers_1 = require("./helpers");
const InboundMessageContext_1 = require("./models/InboundMessageContext");
let MessageReceiver = class MessageReceiver {
    constructor(config, envelopeService, transportService, messageSender, connectionRepository, dispatcher, didRepository) {
        this.inboundTransports = [];
        this.config = config;
        this.envelopeService = envelopeService;
        this.transportService = transportService;
        this.messageSender = messageSender;
        this.connectionRepository = connectionRepository;
        this.dispatcher = dispatcher;
        this.didRepository = didRepository;
        this.logger = this.config.logger;
    }
    registerInboundTransport(inboundTransport) {
        this.inboundTransports.push(inboundTransport);
    }
    /**
     * Receive and handle an inbound DIDComm message. It will decrypt the message, transform it
     * to it's corresponding message class and finally dispatch it to the dispatcher.
     *
     * @param inboundMessage the message to receive and handle
     */
    async receiveMessage(inboundMessage, session) {
        this.logger.debug(`Agent ${this.config.label} received message`);
        if (this.isPlaintextMessage(inboundMessage)) {
            await this.receivePlaintextMessage(inboundMessage);
        }
        else {
            await this.receiveEncryptedMessage(inboundMessage, session);
        }
    }
    async receivePlaintextMessage(plaintextMessage) {
        const message = await this.transformAndValidate(plaintextMessage);
        const messageContext = new InboundMessageContext_1.InboundMessageContext(message, {});
        await this.dispatcher.dispatch(messageContext);
    }
    async receiveEncryptedMessage(encryptedMessage, session) {
        const decryptedMessage = await this.decryptMessage(encryptedMessage);
        const { plaintextMessage, senderKey, recipientKey } = decryptedMessage;
        const connection = await this.findConnectionByMessageKeys(decryptedMessage);
        this.logger.info(`Received message with type '${plaintextMessage['@type']}' from connection ${connection === null || connection === void 0 ? void 0 : connection.id} (${connection === null || connection === void 0 ? void 0 : connection.theirLabel})`, plaintextMessage);
        const message = await this.transformAndValidate(plaintextMessage, connection);
        // We want to save a session if there is a chance of returning outbound message via inbound transport.
        // That can happen when inbound message has `return_route` set to `all` or `thread`.
        // If `return_route` defines just `thread`, we decide later whether to use session according to outbound message `threadId`.
        if (senderKey && recipientKey && message.hasAnyReturnRoute() && session) {
            this.logger.debug(`Storing session for inbound message '${message.id}'`);
            const keys = {
                recipientKeys: [senderKey],
                routingKeys: [],
                senderKey: recipientKey,
            };
            session.keys = keys;
            session.inboundMessage = message;
            // We allow unready connections to be attached to the session as we want to be able to
            // use return routing to make connections. This is especially useful for creating connections
            // with mediators when you don't have a public endpoint yet.
            session.connection = connection !== null && connection !== void 0 ? connection : undefined;
            this.transportService.saveSession(session);
        }
        const messageContext = new InboundMessageContext_1.InboundMessageContext(message, {
            // Only make the connection available in message context if the connection is ready
            // To prevent unwanted usage of unready connections. Connections can still be retrieved from
            // Storage if the specific protocol allows an unready connection to be used.
            connection: (connection === null || connection === void 0 ? void 0 : connection.isReady) ? connection : undefined,
            senderVerkey: senderKey,
            recipientVerkey: recipientKey,
        });
        await this.dispatcher.dispatch(messageContext);
    }
    /**
     * Decrypt a message using the envelope service.
     *
     * @param message the received inbound message to decrypt
     */
    async decryptMessage(message) {
        try {
            return await this.envelopeService.unpackMessage(message);
        }
        catch (error) {
            this.logger.error('Error while decrypting message', {
                error,
                encryptedMessage: message,
                errorMessage: error instanceof Error ? error.message : error,
            });
            throw error;
        }
    }
    isPlaintextMessage(message) {
        if (typeof message !== 'object' || message == null) {
            throw new error_1.AriesFrameworkError('Invalid message received. Message should be object');
        }
        // If the message does have an @type field we assume the message is in plaintext and it is not encrypted.
        return '@type' in message;
    }
    async transformAndValidate(plaintextMessage, connection) {
        let message;
        try {
            message = await this.transformMessage(plaintextMessage);
            await this.validateMessage(message);
        }
        catch (error) {
            if (connection)
                await this.sendProblemReportMessage(error.message, connection, plaintextMessage);
            throw error;
        }
        return message;
    }
    async findConnectionByMessageKeys({ recipientKey, senderKey, }) {
        // We only fetch connections that are sent in AuthCrypt mode
        if (!recipientKey || !senderKey)
            return null;
        let connection = null;
        // Try to find the did records that holds the sender and recipient keys
        const ourDidRecord = await this.didRepository.findByVerkey(recipientKey);
        // If both our did record and their did record is available we can find a matching did record
        if (ourDidRecord) {
            const theirDidRecord = await this.didRepository.findByVerkey(senderKey);
            if (theirDidRecord) {
                connection = await this.connectionRepository.findSingleByQuery({
                    did: ourDidRecord.id,
                    theirDid: theirDidRecord.id,
                });
            }
            else {
                connection = await this.connectionRepository.findSingleByQuery({
                    did: ourDidRecord.id,
                });
                // If theirDidRecord was not found, and connection.theirDid is set, it means the sender is not authenticated
                // to send messages to use
                if (connection && connection.theirDid) {
                    throw new error_1.AriesFrameworkError(`Inbound message senderKey '${senderKey}' is different from connection did`);
                }
            }
        }
        // If no connection was found, we search in the connection record, where legacy did documents are stored
        if (!connection) {
            connection = await this.connectionRepository.findByVerkey(recipientKey);
            // Throw error if the recipient key (ourKey) does not match the key of the connection record
            if (connection && connection.theirKey !== null && connection.theirKey !== senderKey) {
                throw new error_1.AriesFrameworkError(`Inbound message senderKey '${senderKey}' is different from connection.theirKey '${connection.theirKey}'`);
            }
        }
        return connection;
    }
    /**
     * Transform an plaintext DIDComm message into it's corresponding message class. Will look at all message types in the registered handlers.
     *
     * @param message the plaintext message for which to transform the message in to a class instance
     */
    async transformMessage(message) {
        // replace did:sov:BzCbsNYhMrjHiqZDTUASHg;spec prefix for message type with https://didcomm.org
        messageType_1.replaceLegacyDidSovPrefixOnMessage(message);
        const messageType = message['@type'];
        const MessageClass = this.dispatcher.getMessageClassForType(messageType);
        if (!MessageClass) {
            throw new problem_reports_1.ProblemReportError(`No message class found for message type "${messageType}"`, {
                problemCode: problem_reports_1.ProblemReportReason.MessageParseFailure,
            });
        }
        // Cast the plain JSON object to specific instance of Message extended from AgentMessage
        return JsonTransformer_1.JsonTransformer.fromJSON(message, MessageClass);
    }
    /**
     * Validate an AgentMessage instance.
     * @param message agent message to validate
     */
    async validateMessage(message) {
        try {
            await MessageValidator_1.MessageValidator.validate(message);
        }
        catch (error) {
            this.logger.error(`Error validating message ${message.type}`, {
                errors: error,
                message: message.toJSON(),
            });
            throw new problem_reports_1.ProblemReportError(`Error validating message ${message.type}`, {
                problemCode: problem_reports_1.ProblemReportReason.MessageParseFailure,
            });
        }
    }
    /**
     * Send the problem report message (https://didcomm.org/notification/1.0/problem-report) to the recipient.
     * @param message error message to send
     * @param connection connection to send the message to
     * @param plaintextMessage received inbound message
     */
    async sendProblemReportMessage(message, connection, plaintextMessage) {
        if (plaintextMessage['@type'] === problem_reports_1.ProblemReportMessage.type) {
            throw new error_1.AriesFrameworkError(message);
        }
        const problemReportMessage = new problem_reports_1.ProblemReportMessage({
            description: {
                en: message,
                code: problem_reports_1.ProblemReportReason.MessageParseFailure,
            },
        });
        problemReportMessage.setThread({
            threadId: plaintextMessage['@id'],
        });
        const outboundMessage = helpers_1.createOutboundMessage(connection, problemReportMessage);
        if (outboundMessage) {
            await this.messageSender.sendMessage(outboundMessage);
        }
    }
};
MessageReceiver = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [AgentConfig_1.AgentConfig,
        EnvelopeService_1.EnvelopeService,
        TransportService_1.TransportService,
        MessageSender_1.MessageSender,
        connections_1.ConnectionRepository,
        Dispatcher_1.Dispatcher,
        DidRepository_1.DidRepository])
], MessageReceiver);
exports.MessageReceiver = MessageReceiver;
//# sourceMappingURL=MessageReceiver.js.map