"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DidPeer = void 0;
const sha256_1 = require("@stablelib/sha256");
const class_transformer_1 = require("class-transformer");
const utils_1 = require("../../../../utils");
const Key_1 = require("../../domain/Key");
const key_type_1 = require("../../domain/key-type");
const parse_1 = require("../../domain/parse");
const peerDidNumAlgo2_1 = require("./peerDidNumAlgo2");
const PEER_DID_REGEX = new RegExp('^did:peer:(([01](z)([1-9a-km-zA-HJ-NP-Z]{5,200}))|(2((.[AEVID](z)([1-9a-km-zA-HJ-NP-Z]{5,200}))+(.(S)[0-9a-zA-Z=]*)?)))$');
function getNumAlgoFromPeerDid(did) {
    return Number(did[9]);
}
class DidPeer {
    constructor({ didDocument, did }) {
        const parsed = parse_1.parseDid(did);
        if (!this.isValidPeerDid(did)) {
            throw new Error(`Invalid peer did '${did}'`);
        }
        this.parsedDid = parsed;
        this._didDocument = didDocument;
    }
    static fromKey(key) {
        const did = `did:peer:0${key.fingerprint}`;
        return new DidPeer({ did });
    }
    static fromDid(did) {
        return new DidPeer({
            did,
        });
    }
    static fromDidDocument(didDocument, numAlgo) {
        if (!numAlgo && didDocument.id.startsWith('did:peer:')) {
            numAlgo = getNumAlgoFromPeerDid(didDocument.id);
        }
        if (!numAlgo) {
            throw new Error('Could not determine numAlgo. The did document must either have a full id property containing the numAlgo, or the numAlgo must be provided as a separate property');
        }
        if (numAlgo === 1 /* GenesisDoc */) {
            // FIXME: We should do this on the JSON value of the did document, as the DidDocument class
            // adds a lot of properties and default values that will mess with the hash value
            // Remove id from did document as the id should be generated without an id.
            const didDocumentBuffer = utils_1.JsonEncoder.toBuffer(Object.assign(Object.assign({}, didDocument.toJSON()), { id: undefined }));
            // TODO: we should improve the buffer/multibase/multihash API.
            const didIdentifier = utils_1.BufferEncoder.toUtf8String(utils_1.MultiBaseEncoder.encode(utils_1.Buffer.from(utils_1.MultiHashEncoder.encode(sha256_1.hash(didDocumentBuffer), 'sha2-256')), 'base58btc'));
            const did = `did:peer:1${didIdentifier}`;
            return new DidPeer({ did, didDocument });
        }
        else if (numAlgo === 2 /* MultipleInceptionKeyWithoutDoc */) {
            const did = peerDidNumAlgo2_1.didDocumentToNumAlgo2Did(didDocument);
            return new DidPeer({ did });
        }
        else {
            throw new Error(`Unsupported numAlgo: ${numAlgo}. Not all peer did methods support parsing a did document`);
        }
    }
    get did() {
        return this.parsedDid.did;
    }
    get numAlgo() {
        // numalgo is the first digit of the method specific identifier
        return Number(this.parsedDid.id[0]);
    }
    get identifierWithoutNumAlgo() {
        return this.parsedDid.id.substring(1);
    }
    isValidPeerDid(did) {
        const isValid = PEER_DID_REGEX.test(did);
        return isValid;
    }
    get didDocument() {
        // Method 1 (numAlgo 0)
        if (this.numAlgo === 0 /* InceptionKeyWithoutDoc */) {
            const key = Key_1.Key.fromFingerprint(this.identifierWithoutNumAlgo);
            const { getDidDocument } = key_type_1.getKeyDidMappingByKeyType(key.keyType);
            return getDidDocument(this.parsedDid.did, key);
        }
        // Method 2 (numAlgo 1)
        else if (this.numAlgo === 1 /* GenesisDoc */) {
            if (!this._didDocument) {
                throw new Error('No did document provided for method 1 peer did');
            }
            // Clone the document, and set the id
            const didDocument = class_transformer_1.instanceToInstance(this._didDocument);
            didDocument.id = this.did;
            return didDocument;
        }
        // Method 3 (numAlgo 2)
        else if (this.numAlgo === 2 /* MultipleInceptionKeyWithoutDoc */) {
            const didDocument = peerDidNumAlgo2_1.didToNumAlgo2DidDocument(this.parsedDid.did);
            return didDocument;
        }
        throw new Error(`Unsupported numAlgo '${this.numAlgo}'`);
    }
}
exports.DidPeer = DidPeer;
//# sourceMappingURL=DidPeer.js.map